package org.enhydra.barracuda.contrib.dbroggisch.display.filters;

import java.util.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import org.enhydra.barracuda.contrib.dbroggisch.display.filters.dtd.*;
import java.lang.reflect.Field;
import org.apache.log4j.*;

public class DateFilter implements Filter {
    private static Logger logger = Logger.getLogger(DateFilter.class.getName());

		private String formatString;
		private DateFormat dateFormat;

		private Locale loc;


		public DateFilter(String formatString, Locale loc) {
				if(loc == null) {
						loc = Locale.getDefault();
				} else {
						this.loc = loc;
				}
				this.formatString = formatString;				
		}

		public DateFilter(String formatString) {
				this(formatString, null);
		}

		public DateFilter() {
				this(null);
		}

    /**
     *
     * @param param1 <description>
     * @return <description>
     */
		public Object filter(Object obj, FilterContext context)
				throws FilterException
		{
				if(obj == null) {
						return null;
				}
				if(obj instanceof java.util.Date) {
						if(dateFormat == null) {
								dateFormat = new SimpleDateFormat(formatString, loc);
						}
						return dateFormat.format((java.util.Date)obj);
				} else {
						throw new FilterException("Not a java.util.Date");
				}
		}


		public Filter configure(Object obj)
				throws FilterException 
		{
				org.enhydra.barracuda.contrib.dbroggisch.display.filters.dtd.Date xmlObj = (org.enhydra.barracuda.contrib.dbroggisch.display.filters.dtd.Date)obj;
				this.formatString = xmlObj.getValue();
				String locale = xmlObj.getLocale();
				if(locale != null && !locale.trim().equals(""))	{
						try {
								Field f = java.util.Locale.class.getDeclaredField(locale);
								Locale loc = (Locale)f.get(null);
								String [] ar = new String[2];
								if(logger.isDebugEnabled()) {
										logger.debug("Setting locale " + locale);
								}
								this.loc = loc;
						} 
						catch(NoSuchFieldException ex) {						
								logger.error(ex.getMessage());
								ex.printStackTrace();
								throw new FilterException("Error configuring DateFilter");
						}
						catch(IllegalAccessException ex) {
								logger.error(ex.getMessage());						
								ex.printStackTrace();				
								throw new FilterException("Error configuring DateFilter");
						}
				}
				return this;

		}
		
}
