package org.enhydra.barracuda.contrib.dbroggisch.display.filters;

import java.util.*;
import org.apache.log4j.Logger;
import org.enhydra.barracuda.contrib.dbroggisch.display.filters.dtd.*;


/**
 * This class executes the <code>filter</code>-method subsequently for all
 * ist child-filters. The first one not to return <b>null</b> or throwing
 * a <code>FilterException</code> wins.
 *
 * @author <a href="mailto:deets@web.de">Diez Roggisch</a>
 * @version 1.0
 */
public class SwitchFilter implements Filter {
	static Logger logger = Logger.getLogger(SwitchFilter.class.getName());
	Filter [] filters;


	public SwitchFilter() {
	}

	public SwitchFilter(Filter [] fs) {
		filters = fs;
	}

	public SwitchFilter(List fs) {
		this((Filter[])fs.toArray(new Filter[1]));
	}



	/* The method invokes the filter of all given filters 
	 * from first to last. The first one not returning null or throwing 
	 * an exception wins.
	 *
	 * @param value to be filtered
	 * @return The first not-null value of the 
	 * @exception org.enhydra.barracuda.contrib.dbroggisch.repopulation.FilterException <description>
	 */
	public Object filter(Object value, FilterContext context) 
		throws FilterException
	{
		Object res = null;
		for(int i = 0; i < filters.length; i++) {
			try {
				res = filters[i].filter(value, context);
				if(logger.isDebugEnabled()) {
					logger.debug("Switch result: " + res);
				}
			} catch(FilterException ex) {
			}
			if(res != null) {
				break;
			}
		}
		return res;
	}

	public Filter configure(Object obj)
		throws FilterException {
		Switch fw = (Switch)obj;
		List childs = FilterFactory.accumulateChilds(fw);
		filters = new Filter[childs.size()];
		int i = 0;
		for(Iterator it = childs.iterator(); it.hasNext();) {
			Object decl = it.next();
			filters[i] = FilterFactory.createFilter(decl.getClass());
			filters[i++].configure(decl);
		}
		return this;
	}


}
