/*
 * $Id: AbstractPage.java,v 1.7 2003/05/08 16:37:43 jacobk Exp $
 */
package org.enhydra.barracuda.contrib.dbroggisch.page;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import javax.servlet.ServletException;
import org.w3c.dom.Document;
import org.enhydra.barracuda.core.comp.BComponent;
import org.enhydra.barracuda.core.comp.ViewContext;
import org.enhydra.barracuda.core.event.BaseEvent;
import org.enhydra.barracuda.core.event.ControlEventContext;
import org.enhydra.barracuda.core.event.DispatchQueue;
import org.enhydra.barracuda.core.event.EventException;
import org.enhydra.barracuda.core.event.ViewEventContext;
import org.enhydra.barracuda.core.event.helper.DefaultViewHandler;
import org.apache.log4j.Logger;


/**
 * @author <a href="mailto:deets@web.de">Diez B. Roggisch</a>
 * @version 1.0
 */
public abstract class AbstractPage extends DefaultViewHandler implements Page {

    private static final Logger logger = Logger.getLogger(AbstractPage.class.getName());
    private List dEvents;

    /**
     * Describe <code>setDefaultEvents</code> method here.
     *
     * @param events a <code>List</code> value
     */
    public void setDefaultEvents(List events) {
        dEvents = events;
    }

    /**
     * Describe <code>createDefaultModels</code> method here.
     *
     * @param ctx a <code>ControlEventContext</code> value
     */
    public void createDefaultModels(ControlEventContext ctx) {
        filterEvents(ctx, dEvents);
    }

    /**
     * Return a Document instance. This is called once before the rendering takes place and
     * should return a new instance.
     *
     * @param iLocale the <code>Locale</code> of the client for locale specific documents
     * @return a <code>Document</code> value
     */
    public abstract Document getDocument(Locale iLocale);

    // handleviewevent is functionality that we don't necessarily want to throw
    // away.. but the whole render/handleviewevent thing is confusing. so we hide it here.
    // overriding the render method gives the old behavior.
    /**
     * Describe <code>handleViewEvent</code> method here.
     *
     * @param root a <code>BComponent</code> value
     * @return a <code>Document</code> value
     * @exception EventException if an error occurs
     * @exception ServletException if an error occurs
     * @exception IOException if an error occurs
     */
    public Document handleViewEvent(BComponent root)
            throws EventException, ServletException, IOException {
        ViewContext ctx = getViewContext();
        Locale iLocale = ctx.getViewCapabilities().getClientLocale();
        if (logger.isDebugEnabled()) logger.debug("Locale sent by the client is: " + iLocale);
        Document doc = getDocument(iLocale);
        if (logger.isInfoEnabled()) logger.info("Created document " + doc.getClass().getName());
        render(root, ctx, doc);
        return doc;
    }

    /**
     * Describe <code>render</code> method here.
     *
     * @param vec a <code>ViewEventContext</code> value
     * @exception EventException if an error occurs
     * @exception ServletException if an error occurs
     * @exception IOException if an error occurs
     */
    public void render(ViewEventContext vec)
            throws EventException, ServletException, IOException {
        handleViewEvent(vec);
    }

    /**
     * Overload this method to perform the actual process of
     * rendering.
     *
     * @param root a <code>BComponent</code> value you have to add your BComponents to.
     * @param vec a <code>ViewContext</code> value
     * @param doc a <code>Document</code> value which is obtained by calling getDocument(Locale)
     * @exception EventException if an error occurs
     * @exception ServletException if an error occurs
     * @exception IOException if an error occurs
     */
    public abstract void render(BComponent root, ViewContext vec, Document doc) 
            throws EventException, ServletException, IOException;

    /**
     * Describe <code>filterEvents</code> method here.
     *
     * @param ctx a <code>ControlEventContext</code> value
     * @param defaultEvents a <code>List</code> value
     */
    public static void filterEvents(ControlEventContext ctx, List defaultEvents) {
        if (defaultEvents != null) {
            BaseEvent event = ctx.getEvent();

            for (Iterator it = defaultEvents.iterator(); it.hasNext();) {
                if (((BaseEvent)it.next()).equals(event)) {
                    if (logger.isDebugEnabled()) logger.debug("Removing event " + event.getClass().getName() + " from the queue");
                    it.remove();
                }          
            }

            DispatchQueue queue = ctx.getQueue();

            for (Iterator it = defaultEvents.iterator(); it.hasNext();) {
                //add remaining events (less the current event) back to the queue
                queue.addEvent((BaseEvent)it.next());
            }       
        }
    }

}
