/*
 * $Id: ExampleBasePage.java,v 1.2 2003/05/08 16:37:43 jacobk Exp $
 */
package org.enhydra.barracuda.contrib.dbroggisch.examples.view.pages;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import javax.servlet.ServletException;
import org.w3c.dom.Document;
import org.enhydra.xml.io.OutputOptions;
//import org.enhydra.xml.xmlc.XMLCStdFactory;
import org.enhydra.barracuda.core.comp.BComponent;
import org.enhydra.barracuda.core.comp.BTemplate;
import org.enhydra.barracuda.core.comp.DefaultTemplateView;
import org.enhydra.barracuda.core.comp.ViewContext;
import org.enhydra.barracuda.core.event.ControlEventContext;
import org.enhydra.barracuda.core.event.EventException;
import org.enhydra.barracuda.core.util.dom.DefaultDOMLoader;
import org.enhydra.barracuda.core.util.dom.DefaultDOMWriter;
import org.enhydra.barracuda.core.util.dom.DOMWriter;
import org.enhydra.barracuda.contrib.dbroggisch.display.IterativeListModel;
import org.enhydra.barracuda.contrib.dbroggisch.page.AbstractPage;
import org.enhydra.barracuda.contrib.dbroggisch.page.Page;
import org.enhydra.barracuda.contrib.dbroggisch.page.SessionPage;
import org.apache.log4j.Logger;

import org.enhydra.barracuda.contrib.dbroggisch.examples.controller.events.*;
import org.enhydra.barracuda.contrib.dbroggisch.examples.view.xmlc.*;


public class ExampleBasePage extends AbstractPage implements SessionPage {

    private static final Logger logger = Logger.getLogger(ExampleBasePage.class.getName());

    public static class State {
        private State() {}
    }

    public static final String NAVIGATION_MODEL_NAME = "NavigationModel";

    public static final State REPOPULATION = new State();
    public static final State PREFIXMAPPING = new State();
    public static final State FILEUPLOAD = new State();
    private State _state = REPOPULATION;

    List _menupoints;

    public void setMenuPoints(List menupoints) {
        _menupoints = menupoints;
    }

    public void setState(State state) {
        _state = state;
    }

    public State getState() {
        return _state;
    }

    public ExampleBasePage() {
        List dEvents = new ArrayList();
        dEvents.add(new NavigationBar());       
        setDefaultEvents(dEvents);
    }

    public void handOver(ControlEventContext ctx, Page p){
        createDefaultModels(ctx);
    }

    public void render(BComponent root, ViewContext vec, Document pDoc)
            throws EventException, ServletException, IOException {
        NavigationHTML doc = (NavigationHTML)pDoc;
        BTemplate navComp = new BTemplate();

        navComp.setView(new DefaultTemplateView(doc.getElementNavigationHook()));

        if(_menupoints == null) return;

        navComp.addModel(new IterativeListModel(NAVIGATION_MODEL_NAME, _menupoints) {
                public Object getItem(String key) {
                    if("MENUPOINT".equals(key)) {
                        return _current;
                    }
                    return "UNDEFINED";
                }
            });

        root.addChild(navComp);     
    }

    public DOMWriter getDOMWriter() {
        OutputOptions oo = DefaultDOMWriter.getDefaultOutputOptions(getDocument(getViewContext().getViewCapabilities().getClientLocale()));

        //add doctype - need to do here because XMLC can't yet add the one embedded in the html to the DOM
        oo.setOmitDocType(false);
        oo.setPublicId("-//W3C//DTD HTML 4.01 Transitional//EN");
        oo.setSystemId("http://www.w3.org/TR/html401/loose.dtd");

        //now render the DOM
        return new DefaultDOMWriter(oo);
    }

    public Document getDocument(Locale iLocale) {
        //return new XMLCStdFactory(null, null).create(NavigationHTML.class);

        Document doc = null;
        //load the localized DOM template
        try {doc = DefaultDOMLoader.getGlobalInstance().getDOM(NavigationHTML.class, iLocale);}
        catch (IOException ioe) {logger.fatal ("Fatal Error loading DOM template:", ioe);}
        return doc;
    }

}
