/**
 * $Id: DataObjectSaveHandler.java,v 1.3 2003/08/14 04:10:57 jacobk Exp $
 */
package org.enhydra.barracuda.contrib.sam.data;

import java.util.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.contrib.sam.xmlform.*;
import org.enhydra.barracuda.contrib.sam.models.*;
import org.apache.log4j.*;

public abstract class DataObjectSaveHandler extends DefaultBaseEventListener implements GetConfig {
    
    protected static Logger logger = Logger.getLogger(DataObjectSaveHandler.class.getName());
    
    public abstract Config getConfig();
    
    protected Config config;
    protected XmlFormMap xfm;
    protected DataObject dataObj;
    
    public void handleControlEvent(ControlEventContext context) throws EventException, javax.servlet.ServletException, java.io.IOException {
        BaseEvent event = context.getEvent();
        HttpServletRequest request = context.getRequest();
        HttpSession session = request.getSession();
        
        config = getConfig();
        
        if (ErrorModel.exists(context)) return;
        logger.info("Saving User");
        xfm = (XmlFormMap) context.getState(config.getMapName());
        String oid = xfm.getStringVal(config.getOidName());
        logger.info("oid is " + oid);
        try {
            // iterate over all form elements any change underlying dataobject
            dataObj = config.getDataObject(oid);
            Map elements = xfm.getElements();
            Iterator iter = elements.entrySet().iterator();
            while (iter.hasNext()) {
                Map.Entry e = (Map.Entry)iter.next();
                String key = (String)e.getKey();
                if (logger.isDebugEnabled()) logger.debug("key " + key);
                FormElement fe = (FormElement)e.getValue();
                Object oldData = fe.getVal();
                Object newData = modifyFormData(key, oldData);
                logger.debug("old: " + oldData + " new: " + newData);
                dataObj.set(key, newData);
            }
            dataObj.save(null);
            logger.debug("oldoid is " + oid);
            String newoid = (String)dataObj.get(config.getOidName());
            logger.debug("newoid is " + oid);


				// sam, 2003-08-05 commented out next 4 lines, otherwise save&email fails (project cockpit...)
            // create a new formmap and store it in event's context for usage during rendering
            //xfm = new XmlFormMap(context, config.getXmlFormName(), config.getFormName(), dataObj);
				//logger.debug("oid from formmap:" + xfm.getStringVal(config.getOidName()));
            //context.putState(config.getMapName(), xfm);
            
        } catch (DataObjectException ex) {
            ErrorModel.create(context, ex);
            logger.error(ex.getMessage(),ex);
        }
    }
    
    /**
     * gives subclasses a hook to change the form data before it is saved
     */
    protected Object modifyFormData(String key, Object value) {
        return value;
    }
}
