package org.enhydra.barracuda.contrib.sam.data;

import java.util.*;
//import java.util.regex.*;
import gnu.regexp.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.plankton.*;

/** This validator ensures that a value matches a given regexp.
 * For compatibility with JDK 1.3, all references to java.util.regex are replaced by gnu.regexp
 */
public class RegexValidator extends DefaultFormValidator {
   
	 // JDK 1.4: uncomment the next line and comment the following line out 
    //protected Pattern re;
	 protected RE re;
    
    /**
     * Public constructor.
     *
     * @param re Pattern to be matched
     */
	 // JDK 1.4: uncomment the next line and comment the following line out 
    //public RegexValidator(Pattern re ) {
    public RegexValidator(RE re ) {
        super();
        this.re = re;
    }
    
    /**
     * Public constructor.
     *
     * @param re Pattern to be matched
     * @param errmsg the message associated with this error
     */
	 // JDK 1.4: uncomment the next line and comment the following line out 
    //public RegexValidator(Pattern re, String ierrmsg) {
    public RegexValidator(RE re, String ierrmsg) {
        super(ierrmsg);
        this.re = re;
    }
    
    /**
     * Validate a FormElement to make sure a given pattern is matched
     * Validation is only supported for FormType.STRING
     *
     * @param element the form element to be validated
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
        if (this.isNull(val, element))
            return;
        
        if (element==null) throw new ValidationException(val, "Object val:"+val+" is associated with a null FormElement");
        if (!element.getType().equals(FormType.STRING)) throw new ValidationException(val, "Unsupported validation: "+val+" is not of FormType.String and cannot be validated by this validator");
        
        String s = val.toString();
	 // JDK 1.4: uncomment the next line and comment the following line out 
        //if (!re.matcher(s).matches()) {
        if (!re.isMatch(s)) {
	 // JDK 1.4: uncomment the next line and comment the following line out 
            //throw this.generateException(element, deferExceptions, "{" + val + "}" + " does not match Pattern " + re.pattern());
            throw this.generateException(element, deferExceptions, "{" + val + "}" + " does not match Pattern " + re.toString());
        }
    }
    
}
