package org.enhydra.barracuda.contrib.sam.models;

import org.enhydra.barracuda.core.comp.*;
import org.apache.log4j.*;
import java.util.*;

/** 
 * Assume a "supermodel" containing a List of Lists (a 2 dimensional structure). 
 * An outer ("enclosing") model iterates over the the 1st dimension. The IterativeSubModel
 * asks the enclosing model for its current value and iterates over the 2nd dimension
 * @author Stefan Armbruster
 * @version $Id: IterativeSubModel.java,v 1.2 2003/07/19 17:48:08 jacobk Exp $
 */
public class IterativeSubModel extends IteratorModel {
    
    protected static Logger logger = Logger.getLogger(IterativeSubModel.class);
    
    /** reference to enclosing model */
    protected TemplateModel enclosing;
    protected String key;
    
    /** initialize the model
     * @param name Name of the model
     * @param enclosing reference to surrounding model
     */
    public IterativeSubModel(String name, String key, TemplateModel enclosing) {
        super(name);
        this.enclosing = enclosing;
        this.key = key;
    }
    
    public void preIterate() {
        if (enclosing==null) {
            logger.error("you didn't specify a valid enclosing model");
            return;
        }
        Object referenceMain = enclosing.getItem(new TemplateDirective(null, null, key, null));
        /*if (referenceMain == null) {
            logger.error("the enclosing model (type: " + enclosing.getClass().getName() + ") doesn't have a " + key + " result for getItem");
            return;
        }*/
        
        // if enclosing model delivers a List, use it. Otherwise, get a list out of the reference by calling getListFromReference
        Collection coll = null;
        if (referenceMain instanceof Collection) {
            logger.debug("List: " + (Collection)referenceMain);
            coll = (Collection)referenceMain;
        } else {
            logger.info("the enclosing model (type: " + enclosing.getClass().getName() + ") doesn't deliver a Collection, so call getListFromReference");
            coll = getListFromReference(referenceMain);
        }
        if (coll == null) {
            logger.error("could not get a list from model " + enclosing.getClass().getName());
        } else {
            setIterator( coll.iterator() );
        }
        super.preIterate();
    }
    
    /** dummy implementation, to be overridden in subclasses */
    protected Collection getListFromReference(Object ref) {
        throw new UnsupportedOperationException("getListFromReference is not yet implemented");
    }
    
    public Object getItem(String key) {
        logger.debug("getting key : " + key);
        return _current;
    }
}
