package org.enhydra.barracuda.contrib.sam.xmlform;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.helper.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.plankton.exceptions.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.apache.log4j.*;

/** Pretty similar class to {@link org.enhydra.barracuda.core.event.helper.BTemplateViewHandler
 * }. The main differences are
 * <ul>
 * <li>A <CODE>ViewContext</CODE> is passed to {@link #getTemplateModels } and
 * {@link #getTemplateClass}
 * <li>provides a hook for modifying a XMLC class <B>before</B> the whole
 * Model/Component rendering is performed
 * </ul>
 * @author Stefan Armbruster
 * @version $Id: ModifyDomViewHandler.java,v 1.2 2003/03/07 06:10:40 jacobk Exp $
 */
public abstract class ModifyDomViewHandler extends DefaultViewHandler {
    
    protected static Logger logger = Logger.getLogger(ModifyDomViewHandler.class.getName());
    
    protected BTemplate templateComp = null;
    
    /** abstract method for getting als required models
     * @param vc the ViewContext of the current RenderEvent
     * @throws EventException
     * @throws ServletException
     * @return either a single model or a {@link java.util.List} of models
     */
    public abstract Object getTemplateModels(ViewContext vc) throws EventException, ServletException, IOException;
    
    /** Return an instance of the template class (must implement Document) */
    public abstract Class getTemplateClass(ViewContext vc) throws EventException, ServletException, IOException;
    
    /**
     * Provide a handle to the underlying BTemplate component
     */
    public BTemplate getBTemplate() {
        return templateComp;
    }
    
    /** Placeholder for DOM-modifications prior to directive mappings. The default
     * implementation is empty.
     * @param page a instance of the class returned by {@link #getTemplateClass}
     * @param vc the context of the current event
     */
    protected void modifyDOM(Document page, ViewContext vc) throws EventException, ServletException, IOException {
    }
    
    /**
     * Generate the view
     */
    public Document handleViewEvent(BComponent root) throws EventException, ServletException, IOException {
        
        ViewContext vc = getViewContext();
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(getTemplateClass(vc),  vc.getViewCapabilities());
        modifyDOM(page, vc);
        
        //create a template component and bind it to the views
        templateComp = new BTemplate();
        Object o = getTemplateModels(vc);
        if (o instanceof TemplateModel) templateComp.addModel((TemplateModel) o);
        else if (o instanceof List) templateComp.addModels((List) o);
        else if (o == null) { }
        else throw new RuntimeException("Fatal err: Model must return either TemplateModel or List");
        templateComp.setView(new DefaultTemplateView(page.getDocumentElement()));
        root.addChild(templateComp);
        
        return page;
    }
}
