package org.enhydra.barracuda.contrib.sam.xmlform;

import java.util.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.contrib.sam.xmlform.dtd.*;
import org.enhydra.barracuda.contrib.sam.xmlform.dtd.Element;
import org.enhydra.barracuda.contrib.sam.xmlform.dtd.Text;
import org.apache.log4j.*;

/** extends ModifyDomViewHandler in order to automatically creation of a XML Form
 * based view.
 */
public abstract class XmlFormViewHandler extends ModifyDomViewHandler {
    
    protected static Logger logger = Logger.getLogger(XmlFormViewHandler.class.getName());

    /** overwrite in subclasses, must return a XmlFormMap defining the fields of the form */
    protected abstract XmlFormMap getXmlForm(ViewContext vc);
    
    /**
     * needed because getTemplateModels does not have a context
     */
    public Object getTemplateModels(ViewContext vc) {
        List models = new Vector();
        XmlFormMap xfm = getXmlForm(vc);
        models.add(xfm);
        
        List cModels = xfm.getChildModels();
        for(Iterator it = cModels.iterator(); it.hasNext();) {
            models.add((TemplateModel)it.next());
        }
        
        return models;
    }
    
    /**
     * helper function for inserting a input element into the dom
     */
    private void insertIntoDOM(Node parent, Node node, boolean useInsertBefore, Node insert) {
        if (useInsertBefore) parent.insertBefore(	node, insert);
        else parent.appendChild(node);
    }
    
    /** change the given Document, copy the templates of the various form types
     * as defined in the XmlFormMap */
    protected void modifyDOM(Document page, ViewContext vc) {
        HttpServletRequest request = vc.getRequest();
        HttpSession session = request.getSession();
        XmlFormMap xmlForm = getXmlForm(vc);
        
        org.w3c.dom.Element textNode = null;
        HTMLElement textNameNode = null;
        HTMLElement textLabelNode = null;
        HTMLInputElement textInputNode = null;
        org.w3c.dom.Element selectNode  = null;
        HTMLElement selectNameNode = null;
        HTMLElement selectLabelNode = null;
        HTMLSelectElement selectInputNode = null;
        org.w3c.dom.Element textareaNode = null;
        HTMLElement textareaNameNode = null;
        HTMLElement textareaLabelNode = null;
        HTMLTextAreaElement textareaInputNode = null;
        
        // get references to the template nodes for varios form field types
        Form form = xmlForm.getForm();
        Iterator iter = form.getRef_idList().iterator();
        while (iter.hasNext()) {
            Ref_id r = (Ref_id)iter.next();
            String type = r.getType();
            if (type.equals("text")) {
                textNode = page.getElementById(r.getId());
                if (r.getName_id() != null) textNameNode = (HTMLElement)page.getElementById(r.getName_id());
                textLabelNode = (HTMLElement)page.getElementById(r.getLabel_id());
                textInputNode = (HTMLInputElement)page.getElementById(r.getInput_id());
            } else if (type.equals("select")) {
                selectNode = page.getElementById(r.getId());
                if (r.getName_id() != null) selectNameNode = (HTMLElement)page.getElementById(r.getName_id());
                selectLabelNode = (HTMLElement)page.getElementById(r.getLabel_id());
                selectInputNode = (HTMLSelectElement)page.getElementById(r.getInput_id());
            } else if (type.equals("textarea")) {
                textareaNode = page.getElementById(r.getId());
                if (r.getName_id() != null) textareaNameNode = (HTMLElement)page.getElementById(r.getName_id());
                textareaLabelNode = (HTMLElement)page.getElementById(r.getLabel_id());
                textareaInputNode = (HTMLTextAreaElement)page.getElementById(r.getInput_id());
            }
        }
        
        // determine insertion mode: insertBefore or appendChild
        boolean isInsertBefore = (form.getInsertBefore() != null);
        Node insert = null;
        Node parent = null;
        if (isInsertBefore) {
            insert = page.getElementById( form.getInsertBefore().getId());
            parent = page.getElementById( form.getInsertBefore().getParent());
        } else parent = page.getElementById( form.getAppendChild().getId());
        
        // all id attributes need to be removed, since id is unique in a document
        if (textNode != null) textNode.removeAttribute("id");
        if (textNameNode != null) textNameNode.removeAttribute("id");
        if (textLabelNode!=null) textLabelNode.removeAttribute("id");
        if (textInputNode!=null) textInputNode.removeAttribute("id");
        if (selectNode!=null) selectNode .removeAttribute("id");
        if (selectNameNode != null) selectNameNode.removeAttribute("id");
        if (selectLabelNode != null) selectLabelNode.removeAttribute("id");
        if (selectInputNode != null) selectInputNode.removeAttribute("id");
        if (textareaNode != null) textareaNode.removeAttribute("id");
        if (textareaNameNode != null) textareaNameNode.removeAttribute("id");
        if (textareaLabelNode != null) textareaLabelNode.removeAttribute("id");
        if (textareaInputNode != null) textareaInputNode.removeAttribute("id");
        
        // all preparation is done, so do the dirty work:
        // for each form element, find the matching form type template and copy it
        iter = form.getElementList().iterator();
        while (iter.hasNext()) {
            Element ele = (Element) iter.next();
            //if (!ele.getName().startsWith(EntryFormMap.PREFIX)) {
            if (ele.getText() != null)  {
                if (logger.isDebugEnabled()) logger.debug("creating a text element");
                Text t = ele.getText();
                if (textNameNode != null) textNameNode.getFirstChild().setNodeValue(ele.getName());
                textLabelNode.getFirstChild().setNodeValue(ele.getLabel());
                if (t.getMaxlength() != null) textInputNode.setMaxLength(Integer.parseInt(t.getMaxlength()));
                else textInputNode.removeAttribute("maxlength");
                textInputNode.setSize(t.getSize());
                textInputNode.setAttribute("class", "Dir::Get_Data." + xmlForm.getName() + "." + ele.getName());
                insertIntoDOM(parent, textNode.cloneNode(true), isInsertBefore, insert);
            } else if (ele.getTextarea() != null)  {
                if (logger.isDebugEnabled()) logger.debug("creating a textarea element");
                Textarea t = ele.getTextarea();
                if (textareaNameNode != null) textareaNameNode.getFirstChild().setNodeValue(ele.getName());
                textareaLabelNode.getFirstChild().setNodeValue(ele.getLabel());
                textareaInputNode.setAttribute("class", "Dir::Get_Data." + xmlForm.getName() + "." + ele.getName());
                if (t.getRows() != null) textareaInputNode.setRows(Integer.parseInt(t.getRows()));
                else  textareaInputNode.removeAttribute("rows");
                if (t.getCols() != null) textareaInputNode.setCols(Integer.parseInt(t.getCols()));
                else  textareaInputNode.removeAttribute("cols");
                insertIntoDOM(parent, textareaNode.cloneNode(true), isInsertBefore, insert);
            } else if (ele.getSelect() != null)  {
                if (logger.isDebugEnabled()) logger.debug("creating a select element");
                if (selectNameNode != null) selectNameNode.getFirstChild().setNodeValue(ele.getName());
                selectLabelNode.getFirstChild().setNodeValue(ele.getLabel());
                selectInputNode.setAttribute("class", "Dir::Get_Data." + xmlForm.getName() + "." + ele.getName());
                logger.debug( "Dir::Get_Data." + xmlForm.getName() + "." + ele.getName());
                insertIntoDOM(parent, selectNode.cloneNode(true), isInsertBefore, insert);
            }
            //}
        }
        if (textNode!=null) parent.removeChild(textNode);
        if (selectNode!=null) parent.removeChild(selectNode);
        if (textareaNode!=null) parent.removeChild(textareaNode);
        
    }
}
