/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ElementFactory.java,v 1.4 2002/03/25 04:44:35 JacobK Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;
import org.w3c.dom.*;

/**
 * This interface defines the methods needed to implement a ListModel.
 */
public interface ElementFactory {

    /**
     * Get the default template element. Once you get the template, you should 
     * always do a cloneNode() on it and then modify the newly created 
     * copy. May return null if no suitable template is available.
     *
     * @return the default template node.
     */
    public Node getDefaultElement();

    /**
     * Get a template element for the key. The key value may be a fully qualified
     * class or interface name, or it may be the tag name that corresponds to 
     * that template. Once you get the template, you should always do a 
     * cloneNode() on it and then modify the copy. May return null if no suitable 
     * template is available.
     *
     * @param key the target key value
     * @return the default template node.
     */
    public Node getElement(String key);

    /**
     * Get a list of all the keys for a given template.
     *
     * @return a list of all the keys for a given template.
     */
    public List getElementKeys();

    /**
     * Get an actual reference to the Document so you can create whatever
     * type of node you wish. This is the least preferred approach to use
     * for creating nodes from a TemplateFactory, because by using the
     * Document to create nodes you will lose any additional markup (ie. 
     * color, font, emphasis, etc) contained in the template for this 
     * particular node.
     *
     * @return the owner document
     */
    public Document getDocument(); 

}