/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: RenderStrategy.java,v 1.2 2003/07/31 15:47:18 jacobk Exp $
 */
package org.enhydra.barracuda.core.comp.renderer;


//csc_110201.1 - added
/**
 * This class defines several basic render strategies.
 * <strong>NEVER_SCRIPT</strong> indicates that we want to avoid scripting at
 * all costs, even if the client supports it. <strong>CUSTOM_SCRIPT</strong> is
 * exactly like NEVER_SCIPT, except that indicates to Barracuda
 * (component renderers, mostly) not to throw exceptions in cases where the
 * rendered markup wouldn't be able to work properly without scripting. One
 * should use CUSTOM_SCRIPT in cases where developer-created custom client-side
 * scripts are already in use and Barracuda shouldn't be interfering with them.
 * <strong>SCRIPT_AS_NEEDED</strong> indicates the renderer should feel free to
 * use scripting as desired (assuming the client supports it of course).  The
 * default render strategy, defined by <strong>DEFAULT_RENDER_STRATEGY</strong,
 * is SCRIPT_AS_NEEDED (unless modified at runtime).
 */
public class RenderStrategy {

    //concrete instances of strategies
    public static final RenderStrategy SCRIPT_AS_NEEDED = new RenderStrategy();
    public static final RenderStrategy NEVER_SCRIPT = new RenderStrategy();
    public static final RenderStrategy CUSTOM_SCRIPT = new RenderStrategy();

    public static RenderStrategy DEFAULT_RENDER_STRATEGY = SCRIPT_AS_NEEDED;

    /**
     * Protected constructor to prevent external instantiation
     */
    protected RenderStrategy() {}
    
}
