/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLTemplateRenderer.java,v 1.11 2003/01/21 05:42:39 jacobk Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;

import org.apache.log4j.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.plankton.*;

/**
 * This class handles the default rendering of a template into an HTML view.
 */
public class HTMLTemplateRenderer extends HTMLComponentRenderer {

    protected static final Logger logger = Logger.getLogger(HTMLTemplateRenderer.class.getName());

    protected EnabledHelper eh = new EnabledHelper();
    
    protected TemplateHelper th = null;
    
    /**
     *
     */
    public HTMLTemplateRenderer() {
        this(null);    
    }
    
    /**
     *
     */
    public HTMLTemplateRenderer(TemplateHelper ith) {
        setTemplateHelper(ith);
    }
    
    /**
     *
     */
    public void setTemplateHelper(TemplateHelper ith) {
        th = ith;
    }
    
    /**
     *
     */
    public TemplateHelper getTemplateHelper() {
        if (th==null) th = new TemplateHelper(this);
        return th;
    }
    
    /**
     * The purpose of this method is for a renderer to provide 
     * a default node (if none exists). This component currently
     * does not provide a default, so it throws an UnsupportedFormatException 
     * instead.
     *
     * @param doc the master Document which can be used to create elements
     *        from scratch
     * @param comp the component that we're dealing with for the current request
     * @param vc the view context for the current request
     * @return a default node (created from scratch)
     * @throws UnsupportedFormatException if the renderer has no default node
     */
//csc_110501.1    public Node createDefaultNode(Document doc, ViewContext vc) throws UnsupportedFormatException;
    public Node createDefaultNode(Document doc, BComponent comp, ViewContext vc) throws UnsupportedFormatException {  //csc_110501.1
        throw new UnsupportedFormatException("Cannot create default node");
    }
    
    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a template component
        if (!(comp instanceof BTemplate)) throw new NoSuitableRendererException("This renderer can only render BTemplate components");
        BTemplate btemplate = (BTemplate) comp;
        Node node = view.getNode();

        //make sure the View implements TemplateView
        if (!(view instanceof TemplateView)) throw new InvalidViewException ("Component is bound to an unsupported View:"+view);
        TemplateView tview = (TemplateView) view;

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //now delegate the actual rendering to the TemplateHelper
        getTemplateHelper().render(btemplate, tview, vc);
        
        //finally, make sure we reflect the components enabled/disabled status
        eh.setEnabled(node, comp.isEnabled());
    }
    
}