/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ApplicationAssembler.java,v 1.5 2001/10/01 22:45:56 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * <p>This interface defines an ApplicationAssembler. This whole concept
 * is still pretty experimental. Basically, the idea is that whatever 
 * implements this interface gets passed a reference to the EventGateway,
 * an XML File name, and a SAX parser class...given this information, the 
 * implementor is free to assemble an event hierarchy, register listeners, 
 * etc.
 *
 * <p>The only implementation right now is found in org.enhydra.barracuda.
 * experimental.assembler.DefaultApplicationAssembler (and this should truly 
 * be viewed as experimental)
 *
 * <p>Look at the source in ApplicationGateway to see how the assembler
 * is invoked when the servlet is intialized.
 */
public interface ApplicationAssembler {
    
    /**
     * Assemble the system, given the root EventGateway and the
     * XML assembly decriptor name. The default parser will be 
     * used.
     *
     * @param rootGateway the root EventGateway
     * @param servletConfig the ServletConfig object
     * @param assemblySourceFile the XML assembly descriptor
     */
    public void assemble(EventGateway irootGateway, ServletConfig iservletConfig, String iassemblySourceFile);
    
    /**
     * Assemble the system, given the root EventGateway, an
     * XML assembly decriptor name, and a specific SAX parser
     * class.
     *
     * @param rootGateway the root EventGateway
     * @param servletConfig the ServletConfig object
     * @param assemblySourceFile the XML assembly descriptor
     * @param parserClass the SAX parser class
     */
    public void assemble(EventGateway irootGateway, ServletConfig iservletConfig, String iassemblySourceFile, String iparserClass);
}
