/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BaseEventListener.java,v 1.4 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;

/**
 * This class defines the methods for Event listeners
 */
public interface BaseEventListener extends EventListener {

    /**
     * Handle all events. This represents a default implementation 
     * that will route the event on to the convenience respective 
     * convenience methods. Notice that we get servlet response and
     * request information from the queue state. This means that if
     * you use a custom event broker, it needs to set this information
     * in the queue.
     *
     * @param event the base event we are handling
     * @param queue the current dispatch queue
     * @throws EventException
     */
    public void handleEvent(EventContext context) throws EventException;

    /**
     * Return true if the event was handled in the handleEvent method. 
     * By default, most implementations will return true for you, so 
     * the only time you'd need to override is if the handler wanted 
     * to specifically indicate that it had NOT handled the event 
     * (ie in logging scenarios).
     *
     * @return true if the event was handled in the handleEvent method
     */
    public boolean isHandled();

    /**
     * Get the ID that identifies this listener. This will typically be the 
     * class name.
     *
     * @return a string that uniquely identifies this listener
     */
    public String getListenerID();

}
