/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: ClientSideRedirectException.java,v 1.6 2002/08/23 21:49:50 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;


/**
 * This class defines a ClientSideRedirectException...throwing this will
 * cause the ApplicationGateway to redirect the browser to the
 * new Event.
 */
public class ClientSideRedirectException extends EventException {

    //private vars
    private String url = null;

    /**
     * The public contructor for ClientSideRedirectException
     *
     * @param newEvent - the newEvent the client browser
     *        should be redirected to
     */
//TODO - I don't think we should actually be using a BaseEvent here in the constructor,
//since by default the ApplicationGateway will only dispatch HttpRequestEvents - csc
    public ClientSideRedirectException (BaseEvent newEvent) {
        setRedirectURL(newEvent.getEventIDWithExtension());
    }

    /**
     * The public contructor for ClientSideRedirectException
     *
     * @param url - the URL the client browser should be
     *         redirected to
     */
    public ClientSideRedirectException (String iurl) {
        setRedirectURL(iurl);
    }

    /**
     * Get the new event that triggered this interrupt
     */
    public String getRedirectURL() {
        return url;
    }

    //csc_082302.2 - added
    /**
     * Set the Redirect URL (normally you don't need to 
     * do this since you simply pass the target URL into the
     * constructor. If, howver, you find the need to modify a
     * redirect after its already been created, you can do it 
     * through this method) 
     */
    public void setRedirectURL(String iurl) {
        url = iurl;
    }
}
