/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EventContext.java,v 1.10 2002/11/02 19:15:52 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;

import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.plankton.data.*;

/**
 * This interface defines the event context. The context 
 * contains information about the event (event, queue, 
 * plus request and response info if appropriate). The
 * context also implements StateMap, so you can use it
 * to pass information between event handlers. The scope 
 * of the context is only for the duration of an event
 * dispatch cycle (ie. from Http Request to Http Response)
 */
public interface EventContext extends StateMap {

    public static final String BASE_EVENT = EventContext.class.getName()+".BaseEvent";
    public static final String DISPATCH_QUEUE = EventContext.class.getName()+".DispatchQueue";
    public static final String VIEW_CAPABILITIES = EventContext.class.getName()+".ViewCapabilities";

    /**
     * @clientCardinality 1 
     */
    /*#ViewCapabilities lnkViewCapabilities;*/

    /**
     * @clientCardinality 1 
     */
    /*#DispatchQueue lnkDispatchQueue;*/

    /**
     * @clientCardinality 1 
     */
    /*#BaseEvent lnkBaseEvent;*/

    /**
     * Get the underlying BaseEvent
     *
     * @return the underlying BaseEvent
     */
    public BaseEvent getEvent();

    /**
     * Get the underlying DispatchQueue
     *
     * @return the underlying DispatchQueue
     */
    public DispatchQueue getQueue();

    /**
     * Get the underlying ViewCapabilities
     */
    public ViewCapabilities getViewCapabilities();
    
    /**
     * The event context must be able to persist its statemap 
     * so that it can reconstruct itself after a ClientSideRedirectException 
     */
//csc_082302.2 - this method should not be exposed via the interface; developers
//should never need to call this function directly
//    public void persistContext();
}
