/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): Diez B. Roggisch
 *
 * $Id: CardinalityValidator.java,v 1.4 2003/01/21 05:42:53 jacobk Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.plankton.*;
import org.apache.log4j.*;

/**
 * This validator ensures that a value is a valid date
 *
 * @author  Diez B. Roggisch <diez.roggisch@artnology.com>
 * @version %I%, %G%
 * @since   1.0
 */
public class CardinalityValidator extends DefaultFormValidator {

    int _min, _max;

    public static final FormValidator ONE = new CardinalityValidator(1, 1);
    public static final FormValidator ONE_TO_MANY = new CardinalityValidator(1, Integer.MAX_VALUE);
    public static final FormValidator ZERO_TO_MANY = new CardinalityValidator(0, Integer.MAX_VALUE);
    public static final FormValidator ZERO_TO_ONE = new CardinalityValidator(0, 1);
    
    /**
     * Public constructor.
     *
     */
    public CardinalityValidator(int min, int max) {
        super();
        _min = min;
        _max = max;
    }

    /**
     * Public constructor.
     *
     */
    public CardinalityValidator(String ierrmsg, int min, int max) {
        super(ierrmsg);
        _min = min;
        _max = max;
    }

    /**
     * Validate a FormElement to make sure the number of values is in the specified range
     */

    public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
//csc_112202.2        Object [] values = element.getMultipleValues();
        Object [] values = element.getVals();   //csc_112202.2

        if (values.length < _min || values.length > _max) {
            if (super.getErrorMessage()!=null) {
                throw this.generateException(element, deferExceptions, super.getErrorMessage());
            } else {
                throw this.generateException(element, deferExceptions, values.toString()
                                             + " with length " + values.length
                                             +  " is not in the valid range [" + _min + ":" + _max + "]");
            }
 
        }
    }

}
