/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: ValidTypeValidator.java,v 1.13 2002/11/02 19:15:52 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.plankton.*;

/**
 * This validator ensures that the original value constitues a
 * valid type
 */
public class ValidTypeValidator extends DefaultFormValidator {

    /**
     * Public constructor.
     *
     * @param errmsg the message associated with this error
     */
    public ValidTypeValidator() {
        super();
    }

    /**
     * Public constructor.
     *
     * @param errorMessage the message associated with this error
     */
    public ValidTypeValidator(String ierrorMessage) {
        super(ierrorMessage);
    }

    /**
     * Public constructor.
     *
     * @param errorMessage the message associated with this error
     * @param deferExceptions true if want generated exceptions to be deferred
     */
//    public ValidTypeValidator(String ierrorMessage, boolean ideferExceptions) {
//        super(ierrorMessage, ideferExceptions);
//    }

    /**
     * Validate a FormElement to make sure that the non-null orig values are valid
     * for the specified element type
     *
     * @param element the form element to be validated
     * @param element the form element that contains the val
     *        to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
        //ilc-02-20-2000 begin changes
        //make sure we validate val
//        Object origVal = element.getOrigVal();
        //ilc-02-20-2000 end changes
        FormType ft = element.getType();
        if (localLogger.isInfoEnabled()) localLogger.info("validating orig val="+val+" is of type:"+ft);
//System.out.println("validating orig val="+origVal+" is of type:"+ft);

        //ilc-02-20-2000 begin changes
        //Do not make judgement calls on null and empty strings
//        if (origVal==null) return;    //this validator makes no judgement calls if the value is null
        //this validator makes no judgement calls if the value is null
        // if (val==null || val.toString().trim().length()<1) return;
        if (isNull(val, element))
          return;
        //ilc-02-20-2000 end changes

        if (element.getParseException()!=null) {
            throw this.generateException(element, deferExceptions, "Invalid value '"+val+"' for form type "+ft);
        }
    }

}
