/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: ParamGateway.java,v 1.14 2003/07/31 15:47:18 jacobk Exp $
 */
package org.enhydra.barracuda.core.helper.servlet;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.log4j.*;

import org.enhydra.barracuda.core.helper.state.*;
import org.enhydra.barracuda.plankton.http.*;

/**
 * <p>The purpose of this servlet is take a set of req parameters
 * and persist them in the users session. Then return the real target
 * url back to the calling application
 */
public class ParamGateway extends HttpServlet {

    protected static final Logger logger = Logger.getLogger(ParamGateway.class.getName());

    //these are intentionally non-final, so that you can programatically
    //change them if need be
    public static String PARAM_TARGET = "pt_";
    public static String PARAM_EXT = ".param_map";


    //-------------------- ParamGateway --------------------------
    /**
     * <p>Handle the default HttpRequest.
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleDefault(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        //persist the params
        if (logger.isDebugEnabled()) {
            logger.debug("Persisting parameter information");
            ServletUtil.showParams(req, logger);
        }
        ParamPersister.persistReqParamState(req);

        //find the client context id
        String cid = req.getParameter("pm_cid");

        //csc_061202.1 - added
        //set the caching hdrs to prevent caching
        resp.setHeader("Cache-Control","no-cache");
        resp.setDateHeader("Expires", System.currentTimeMillis());

        //figure out where we need to go from here by deriving it from
        //the actual URL used to get us here
        String src = req.getRequestURI();
        if (logger.isDebugEnabled()) logger.debug("Request URI:"+src);
        int spos = src.indexOf(PARAM_TARGET);
        int epos = src.indexOf(PARAM_EXT);
        String base = src.substring(spos+PARAM_TARGET.length(), epos);
        String param = src.substring(epos+PARAM_EXT.length());
        String target = URLRewriter.encodeURL(req, resp, base+param);
        if (logger.isDebugEnabled()) logger.debug("Param Target:"+target);
        resp.setContentType("text/html");
        PrintWriter out = resp.getWriter();

        out.println("<html>");
        out.println("  <head>");
        out.println("    <title></title>");
        out.println("    <script type=\"text/javascript\">window.onerror=new Function('return true');</script>");
        out.println("  </head>");
        out.print  ("  <body onload=\"p=document.layers?parentLayer:window.parent;p.jsrsLoaded('");out.print(cid);out.println("');\">");
        out.print  ("    <h3>Redirecting to real target...</h3><p>If you are not automatically redirected, please click <a href=\"");out.print(target);out.println("\" onclick=\"location.replace(this.href);return false;\">here</a></p>");
        out.print  ("    <form name=\"jsrs_Form\"><div style=\"visibility:hidden;\"><textarea name=\"jsrs_Payload\" rows=\"2\" cols=\"20\">");out.print(target);out.println("</textarea></div></form>");
        out.println("  </body>");
        out.println("</html>");

        //out.flush();
        resp.flushBuffer(); //instead of out.flush for Servlet 2.2 compatibility (Craig McClanahan http://w4.metronet.com/~wjm/tomcat/2000/Nov/msg00174.html)
        out.close();
    }


    //-------------------- HTTPServlet ---------------------------
    /**
     * <p>By default the GET request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }

    /**
     * <p>By default the POST request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }

    /**
     * <p>By default the OPTIONS request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doOptions(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }

    /**
     * <p>By default the DELETE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doDelete(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }

    /**
     * <p>By default the PUT request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPut(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }

    /**
     * <p>By default the TRACE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doTrace(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }

}
