/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DOMFactory.java,v 1.5 2003/06/02 19:09:28 jacobk Exp $
 */
package org.enhydra.barracuda.core.util.dom;

import java.io.IOException;
import org.w3c.dom.Document;

/**
 * This interface defines the methods needed to implement a DOMFactory. A dom
 * factory is not meant to be called directly, but by the chosen DOMLoader
 * implementation. As such, which getInstance() method gets called depends
 * upon which method(s) the DOMLoader and the supplied DOMFactory
 * implementation combination supports.
 */
public interface DOMFactory {

    /**
     * Get a new instance of the DOM that is associated with the
     * given class
     *
     * @param clazz the class to be loaded as a DOM object
     * @return the document that most closely corresponds with the requested
     *         class/locale combination
     */
    public Document getInstance(Class clazz) throws IOException;

    /**
     * Load a document directly from file.
     * <p>Note that the behavior of this method depends
     * upon the dom factory implementation being used. Some dom
     * factories may not support this method and just throw an IOException.
     * Those that support this method may require the docPath as an OS-specifc
     * hardcoded path, a path relative to a known hardcoded path, a path to the
     * file within the classloader, or anything else you can imagine. See the
     * javadoc of the dom factory implementation you are using for details.</p>
     *
     * @param docPath the document file to be loaded as a DOM object
     * @return the document that most closely corresponds with the requested
     *         doc path/locale combination
     */
    public Document getInstance(String docPath) throws IOException;

}
