/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: XMLCStdDOMFactory.java,v 1.2 2003/06/02 19:09:28 jacobk Exp $
 */
package org.enhydra.barracuda.core.util.dom;

import java.io.IOException;
import org.w3c.dom.Document;
import org.apache.log4j.Logger;
import org.enhydra.xml.xmlc.XMLObject;
import org.enhydra.xml.xmlc.XMLCStdFactory;


/**
 * XMLC standard implementation of a DOMFactory. This class will load a DOM using
 * XMLCStdFactory.
 */
public class XMLCStdDOMFactory implements DOMFactory {

    /**
     * used for logging
     */
    protected static final Logger logger = Logger.getLogger(XMLCStdDOMFactory.class.getName());

    /**
     * arbitrary object used to synchronize upon
     */
    protected final Object sync = new Object();

    /**
     * used for flagging whether the xmlcFactory has been initialized and
     * avoiding unnecessary synchronization.
     */
    protected boolean initialized = false;

    /**
     * XMLCFactory instance, stored so that it isn't re-created on
     * every request.
     */
    protected XMLCStdFactory xmlcFactory;

    /**
     * Get a new instance of the DOM that is associated with the
     * given class.
     *
     * @param clazz the class to be loaded as a DOM object. In this case, this class
     *        should implement XMLObject or the underlying XMLCFactory will
     *         not be able to instantiate it.
     * @return the document that most closely corresponds with the requested
     *        class/locale combination
     */
    public Document getInstance(Class clazz) throws IOException {
        if (!((XMLObject.class).isAssignableFrom(clazz))) throw new IOException ("Class "+clazz.getName()+" can not be loaded by this DOMFactory because it does not implement XMLOBject");
        initFactory();
        return xmlcFactory.create(clazz);
    }

    /**
     * This method is not supported by this dom factory and will
     * immediately throw an IOException if called!
     */
    public Document getInstance(String docPath) throws IOException {
        throw new IOException("Error: Unimplemented - XMLCStdFactory does not support creation of documents directly from file");
    }

    private void initFactory() {
        if (initialized == false) {
            synchronized (sync) {
                logger.info("initializing an XMLC deferred parsing factory for returning XMLC-generated documents");
                xmlcFactory = new XMLCStdFactory(Thread.currentThread().getContextClassLoader(), null);
                initialized = true;
            }
        }
    }

}
