/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LocalUtilities.java,v 1.13 2003/01/21 05:42:53 jacobk Exp $
 */
package org.enhydra.barracuda.examples.ex1;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.log4j.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.plankton.data.*;
import org.enhydra.barracuda.plankton.http.*;
import org.enhydra.barracuda.examples.ex1.events.*;


/**
 * General utility event handlers which apply to all screens
 * in the app. This really provides "master" controller functionality
 * which gets applied to all events getting dispatched into the system 
 * through the Http gateway.
 */
public class LocalUtilities extends DefaultEventGateway {

    //public constants
    protected static final Logger logger = Logger.getLogger(LocalUtilities.class.getName());

    //this defines the various event handlers
    private ListenerFactory localRequestFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new LocalRequestHandler();}        public String getListenerID() {return getID(LocalRequestHandler.class);}};                

    //private vars
    private LocalRequestEvent lnkLocalRequestEvent;            //added by TJ to show association

    /**
     * Public constructor
     */
    public LocalUtilities() {
        //specify who's interested in what
        specifyLocalEventInterests(localRequestFactory, LocalRequestEvent.class);
    }
    
    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * HttpRequest - Here we redirect to the LoginScreen if necessary.
     * Make sure they aren't trying to dispatch a bogus event, and make sure
     * they have user/pwd info in their session.
     *
     * NOTE: if we were aiming for reusability, we could actually implement
     * this as multiple listeners in standalone classes, allowing a developer 
     * to pick and choose as needed...for now, we just kept it simple and lumped 
     * it all into one handler.
     */
    class LocalRequestHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            HttpServletRequest req = context.getRequest();
            DispatchQueue queue = context.getQueue();

            //figure out which event triggered this request
            logger.debug("Locating the event which triggered this one");
            List procList = queue.listProcessedEvents();
            BaseEvent triggerEvent = (BaseEvent) procList.get(procList.size()-1);
            logger.debug("Trigger event:"+triggerEvent);

            //see if we can validate user/pwd info
            HttpSession session = req.getSession();
            int valid = LoginServices.validateUserPwd(session);
            logger.debug("Checking for valid user/pwd info in the session...result:"+LoginServices.xrefValidity(valid));

            //if the event which triggered this is NOT an instance of 
            //LoginScreen event (meaning they're trying to do something 
            //OTHER than log in) and they don't have valid credentials,
            //redirect them to the LoginScreen
            if (valid!=LoginServices.VALID && (triggerEvent==null || (!(triggerEvent instanceof LoginScreenEvent)))) {
                logger.debug("Redirecting to the LoginScreen because they lack valid credentials");
                throw new ClientSideRedirectException(new GetLoginScreen());
            }
            
            //if the event which triggered this IS a LoginScreen event
            //(meaning they're trying to log in again) and they already
            //have valid login information, just redirect them onto the 
            //main screen
            if (triggerEvent!=null && (triggerEvent instanceof LoginScreenEvent) && valid==LoginServices.VALID) {
                logger.debug("Redirecting to MainScreen since they already have valid credentials");
                
//csc_082302.2_start
                //rather than store the value in the session, we can put it in the 
                //LocalRepository since that structure is persisted across the multiple
                //req-resp cycles that happen when we throw a ClientSideRedirectException.
                //If for some reason the lr and state information stop getting persisted 
                //(ie. if we inadvertantly introdcue a bug), then this auto-login
                //stuff will stop working
//                session.setAttribute(LoginServices.AUTO_LOGIN, ""+true);
                ObjectRepository lr = ObjectRepository.getLocalRepository();
                lr.putState(LoginServices.AUTO_LOGIN, Boolean.TRUE);
//csc_082302.2_end
                throw new ClientSideRedirectException(new GetMainScreen());
            }
        }
    }
}

