/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: StateMap.java,v 1.2 2003/05/28 16:07:09 christianc Exp $
 */
package org.enhydra.barracuda.plankton.data;

import java.io.*;
import java.util.*;

/**
 * <p>This interface defines the methods needed to implement
 * state in an object. By this we mean that an object is
 * capable of carrying state information along with it--you
 * can put properties into the state and then get them back
 * out.
 *
 * <p>Key entities that implement StateMap:
 * <ul>
 *        <li>BaseEvent</li>
 *        <li>EventContext</li>
 *        <li>FormMap</li>
 * </ul>
 */
public interface StateMap {

    /**
     * set a property in this StateMap
     *
     * @param key the key object
     * @param val the value object
     */
    public void putState(Object key, Object val);

    /**
     * get a property in this StateMap
     *
     * @param key the key object
     * @return the value for the given key
     */
    public Object getState(Object key);

    /**
     * remove a property in this StateMap
     *
     * @param key the key object
     * @return the object which was removed
     */
    public Object removeState(Object key);

    /**
     * get a list of the keys for this StateMap
     *
     * @return a list the keys for this StateMap
     */
    public List getStateKeys();

    /**
     * get a copy of the underlying Map that holds
     * the state values
     *
     * @return a copy of the underlying state Map
     */
    public Map getStateValues();

    //csc_052803_2 - added
    /**
     * clear all state information
     */
    public void clearState();
}
