/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: URLRewriter.java,v 1.4 2003/05/16 13:39:39 jacobk Exp $
 */
package org.enhydra.barracuda.plankton.http;

import javax.servlet.*;
import javax.servlet.http.*;


/**
 * This just contains some Servlet utility routines
 */
public class URLRewriter {

    /**
     * This option is used to control whether or not URL rewriting occurs. It 
     * exists to handle bugs in Enhydra 3.x in which invoking the URL rewriting
     * methods causes a bogus value to be returned. If you are using Enhydra 3.x,
     * you will want to set this value to false (you can do this without coding
     * changes via the your assembler xml file).  Defaults to true. To see whether
     * or not your appserver correctly implements URL rewriting, look at 
     * http://localhost/Barracuda/RedirectEx1
     */
    public static boolean REWRITE_URLS = true;
//    public static boolean REWRITE_URLS = false;            //need it false if we're running against Enhydra 3.x
    
    /**
     * Encode a URL if the REWRITE_URLS option is set. Using this option
     * makes it easy to reconfigure Barracuda if your server has problems
     * with URL rewriting (ie. Enhydra 3.x)
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @param url the target url
     * @return an encoded url (unless REWRITE_URLS = false, in which case
     *        it will just return the original url value)
     */
    public static String encodeURL(HttpServletRequest req, HttpServletResponse resp, String url) {
        //jrk_20030508 - should be able to remove the req parameter, but leaving
        //to avoid issues with an interface change.
        if (REWRITE_URLS && resp != null) {
            return resp.encodeURL(url);
        }
        return url;
    }

    /**
     * Encode a redirect URL if the REWRITE_URLS option is set. Using this option
     * makes it easy to reconfigure Barracuda if your server has problems
     * with URL rewriting (ie. Enhydra 3.x)
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @param url the target url
     * @return an encoded url (unless REWRITE_URLS = false, in which case
     *        it will just return the original url value)
     */
    public static String encodeRedirectURL(HttpServletRequest req, HttpServletResponse resp, String url) {
        //jrk_20030508 - should be able to remove the req parameter, but leaving
        //to avoid issues with an interface change.
        if (REWRITE_URLS && resp != null) {
            return resp.encodeRedirectURL(url);
        }
        return url;
    }

}    
