package org.enhydra.barracuda.testbed.workbench.util.gui;


import java.awt.*;
import javax.swing.*;


/**
 * Cursors utility
 */
public class Cursors {

    /**
     * This method basically runs through all the components 
     * in the heirarchy and sets the cursor for them 
     * depending on the value of isWorking.
     *
     * @param cont - The container in question
     * @param isWorking - True if we want to show that we're working
     */
    public static void setWorking(Container cont, boolean isWorking) {
        //eliminate the obvious
        if (cont==null) return;

        //get all the components in this container
        Component comps[] = cont.getComponents();
        setWorking(comps, isWorking);
    }

    /**
     * This method basically runs through all the components 
     * in the heirarchy and sets the cursor for them 
     * depending on the value of isWorking.
     *
     * @param comps - An array of components in questions
     * @param isWorking - True if we want to show that we're working
     */
    public static void setWorking(Component comps[], boolean isWorking) {
        setWorking_private(comps, isWorking);
    }

    /**
     * This is the method that gets called recursively and does all the work
     *
     * @param comps
     * @param isWorking
     */
    public static void setWorking_private(Component comps[], boolean isWorking) {

        //look for specifics
        Cursor defaultCursor = Cursor.getDefaultCursor();
        Cursor handCursor = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
        Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
        for (int i = 0, max = comps.length; i<max; i++) {
            Component comp = comps[i];
            if (comp==null) continue;
            if (isWorking) {
                comp.setCursor(waitCursor);
            } else {
                if (comp instanceof AbstractButton) comp.setCursor(handCursor);
                else if (comp instanceof Button) comp.setCursor(handCursor);
                else comp.setCursor(defaultCursor);
            }

            //Now see if the component happens to be a container. If so, we'll
            //need to iterate through that as well
            if (comp instanceof Container) setWorking((Container) comp, isWorking);
        }
    }


}
