/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestMaxLengthValidator.java,v 1.7 2002/11/22 23:10:23 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.plankton.data.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.testbed.*;


/**
 * This test verifies that MaxLengthValidator works correctly.
 */
public class TestMaxLengthValidator extends ValidatorTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestMaxLengthValidator.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestMaxLengthValidator(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Test String types
     */
    public void testString() {
        //create a form validator and excercise it
        MaxLengthValidator v1 = new MaxLengthValidator(-1);
        MaxLengthValidator v2 = new MaxLengthValidator(0);
        MaxLengthValidator v3 = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.STRING);

        // ilc_022502.1_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 max length
        sm.putState("valid string -1 length", null);
        sm.putState("invalid string -1 length1", "");
        sm.putState("invalid string -1 length1", "   ");
        assertAllValid(v1, sm, FormType.STRING);

        sm = null;
        sm = new DefaultStateMap();
        // we've decided "" is equivalant of null
        //sm.putState("invalid string -1 length1", "");
        sm.putState("invalid string -1 length2", "foo");
        sm.putState("invalid string -1 length3", "foofoo");
        assertAllInvalid(v1, sm, FormType.STRING);

        // Validate 0 max length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid string 0 length1", null);
        sm.putState("valid string 0 length2", "");
        assertAllValid(v2, sm, FormType.STRING);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("Invalid string 0 length1", "f");
        sm.putState("Invalid string 0 length1", "foo");
        sm.putState("invalid string 0 length2", "foofoo");
        assertAllInvalid(v2, sm, FormType.STRING);

        // Validate 5 max length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid string 5 length1", null);
        sm.putState("valid string 5 length2", "");
        sm.putState("valid string 5 length3", "foo");
        sm.putState("valid string 5 length3", "foofo");
        assertAllValid(v3, sm, FormType.STRING);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid string 5 length1", "foofoo");
        assertAllInvalid(v3, sm, FormType.STRING);

        // ilc_022502.1_end

        // ilc_022502.2_start
        /* use state maps instead
        //v1
        assertValid("Error validating null.length()<=-1", v1, el, null);
        assertInvalid("Error invalidating ''.length()<=-1", v1, el, "");
        assertInvalid("Error invalidating foo.length()<=-1", v1, el, "foo");
        assertInvalid("Error invalidating foofoo.length()<=-1", v1, el, "foofoo");
        
        //v2
        assertValid("Error validating null.length()<=0", v2, el, null);
        assertValid("Error validating ''.length()<=0", v2, el, "");
        assertInvalid("Error invalidating foo.length()<=0", v2, el, "foo");
        assertInvalid("Error invalidating foofoo.length()<=0", v2, el, "foofoo");
        
        //v3
        assertValid("Error validating null.length()<=5", v3, el, null);
        assertValid("Error validating ''.length()<=5", v3, el, "");
        assertValid("Error validating foo.length()<=5", v3, el, "foo");
        assertInvalid("Error invalidating foofoo.length()<=5", v3, el, "foofoo");
        */
        // ilc_022502.2_end
    }
    
    /**
     * Test Boolean types - this test ensures that if we pass in a 
     * null FormElement or try to validate the length on a Boolean
     * it will always generate a ValidationException
     */
    public void testBoolean() {
        //create a form validator and excercise it
        MaxLengthValidator v = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.BOOLEAN);
        Boolean bT = new Boolean(true);
        Boolean bF = new Boolean(false);

        // ilc_022502.3_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 max length
        sm.putState("valid bool 5 length", null);
        assertAllValid(v, sm, FormType.BOOLEAN);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid bool 5 length1", bT);
        sm.putState("invalid bool 5 length1", bF);
        assertAllInvalid(v, sm, FormType.BOOLEAN);
        // ilc_022502.3_end

        // ilc_022502.4_start
        // use StateMap to test
        /*
        //v
        assertInvalid("Error invalidating null element", v, null, "foo");
        assertInvalid("Error invalidating Boolean(true)", v, el, bT);
        assertInvalid("Error invalidating Boolean(false)", v, el, bF);
        */
        // ilc_022502.4_end
    }
    
    /**
     * Test Integer types
     */
    public void testInteger() {
        //create a form validator and excercise it
        MaxLengthValidator v1 = new MaxLengthValidator(-1);
        MaxLengthValidator v2 = new MaxLengthValidator(0);
        MaxLengthValidator v3 = new MaxLengthValidator(4);
        DefaultFormElement el = new DefaultFormElement("key", FormType.INTEGER);
        Integer i = new Integer(1234);

        // ilc_022502.5_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 max length
        sm.putState("valid integer -1 length", null);
        assertAllValid(v1, sm, FormType.INTEGER);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid integer -1 length", i);
        assertAllInvalid(v1, sm, FormType.INTEGER);

        // Validate 0 max length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid integer 0 length", null);
        assertAllValid(v2, sm, FormType.INTEGER);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid integer 0 length", i);
        assertAllInvalid(v2, sm, FormType.INTEGER);

        // Validate 5 max length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid integer 5 length1", null);
        sm.putState("valid integer 5 length2", i);
        assertAllValid(v3, sm, FormType.INTEGER);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid integer 0 length", new Integer(12345));
        assertAllInvalid(v3, sm, FormType.INTEGER);
        // ilc_022502.5_end

        // ilc_022502.6_start
        // use statemap instead
        /*
        //v1
        assertValid("Error validating null.length()<=-1", v1, el, null);
        assertInvalid("Error invalidating i.length()<=-1", v1, el, i);
        
        //v2
        assertValid("Error validating null.length()<=0", v2, el, null);
        assertInvalid("Error invalidating i.length()<=0", v2, el, i);
        
        //v3
        assertValid("Error validating null.length()<=5", v3, el, null);
        assertValid("Error validating i.length()<=5", v3, el, i);
        */
        // ilc_022502.6_end
    }

    /**
     * Test Date types - this test ensures that if we try to validate 
     * the length on a Date it will always generate a ValidationException
     */
    public void testDate() {
        //create a form validator and excercise it
        MaxLengthValidator v = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.DATE);
        Date d = new Date();

        // ilc_022502.7_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 max length
        sm.putState("valid date 5 length", null);
        assertAllValid(v, sm, FormType.DATE);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid date 5 length1", d);
        assertAllInvalid(v, sm, FormType.BOOLEAN);
        // ilc_022502.7_end

        // ilc_022502.8_start
        // use StateMap to test
        /*
        //v
        assertValid("Error validating null date", v, el, null);
        assertInvalid("Error invalidating Date", v, el, d);
        */
        // ilc_022502.8_end
    }

    /**
     * Test Long types
     */
    public void testLong() {
        //create a form validator and excercise it
        MaxLengthValidator v1 = new MaxLengthValidator(-1);
        MaxLengthValidator v2 = new MaxLengthValidator(0);
        MaxLengthValidator v3 = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.LONG);
        Long l = new Long(123);

        // ilc_022502.9_start
        // use a StateMap for testing to make sure elements go through mapping
        Long l4 = new Long(1234);
        Long l5 = new Long(12345);
        Long l6 = new Long(123456);

        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 max length
        sm.putState("valid long -1 length", null);
        assertAllValid(v1, sm, FormType.LONG);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid long -1 length1", l);
        assertAllInvalid(v1, sm, FormType.LONG);

        // validate 0 max length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid long 0 length1", null);
        assertAllValid(v2, sm, FormType.LONG);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid long 0 length1", l);
        assertAllInvalid(v2, sm, FormType.LONG);

        // validate 5 max length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid long 5 length1", null);
        sm.putState("invalid long 5 length2", l4);
        sm.putState("invalid long 5 length3", l5);
        assertAllValid(v3, sm, FormType.LONG);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid long 5 length1", l6);
        assertAllInvalid(v3, sm, FormType.LONG);


        // ilc_022502.9_end

        // ilc_022502.10_start
        // use StateMap to test
        /*
        //v1
        assertValid("Error validating null.length()<=-1", v1, el, null);
        assertInvalid("Error invalidating l.length()<=-1", v1, el, l);
        
        //v2
        assertValid("Error validating null.length()<=0", v2, el, null);
        assertInvalid("Error invalidating l.length()<=0", v2, el, l);
        
        //v3
        assertValid("Error validating null.length()<=5", v3, el, null);
        assertValid("Error validating l.length()<=5", v3, el, l);
        */
        // ilc_022502.10_end
    }

    /**
     * Test Short types
     */
    public void testShort() {
        //create a form validator and excercise it
        MaxLengthValidator v1 = new MaxLengthValidator(-1);
        MaxLengthValidator v2 = new MaxLengthValidator(0);
        MaxLengthValidator v3 = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.SHORT);
        Short s = new Short((short)123);

        // ilc_022502.11_start
        // use a StateMap for testing to make sure elements go through mapping

        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 max length
        sm.putState("valid short -1 length", null);
        assertAllValid(v1, sm, FormType.SHORT);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid short -1 length1", s);
        assertAllInvalid(v1, sm, FormType.SHORT);

        // validate 0 max length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid short 0 length1", null);
        assertAllValid(v2, sm, FormType.SHORT);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid short 0 length1", s);
        assertAllInvalid(v2, sm, FormType.SHORT);

        // validate 5 max length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid short 5 length1", null);
        sm.putState("valid short 5 length2", s);
        assertAllValid(v3, sm, FormType.SHORT);


        // ilc_022502.11_end

        // ilc_022502.12_start
        // use StateMap to test
        //v1
        /*
        assertValid("Error validating null.length()<=-1", v1, el, null);
        assertInvalid("Error invalidating s.length()<=-1", v1, el, s);
        
        //v2
        assertValid("Error validating null.length()<=0", v2, el, null);
        assertInvalid("Error invalidating s.length()<=0", v2, el, s);
        
        //v3
        assertValid("Error validating null.length()<=5", v3, el, null);
        assertValid("Error validating s.length()<=5", v3, el, s);
        */
        // ilc_022502.12_end
    }

    /**
     * Test Double types
     */
    public void testDouble() {
        //create a form validator and excercise it
        MaxLengthValidator v1 = new MaxLengthValidator(-1);
        MaxLengthValidator v2 = new MaxLengthValidator(0);
        MaxLengthValidator v3 = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.DOUBLE);
        Double d = new Double(123);

        // ilc_022502.13_start
        // use a StateMap for testing to make sure elements go through mapping

        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 max length
        sm.putState("valid double -1 length", null);
        assertAllValid(v1, sm, FormType.DOUBLE);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid double -1 length1", d);
        assertAllInvalid(v1, sm, FormType.DOUBLE);

        // validate 0 max length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid double 0 length1", null);
        assertAllValid(v2, sm, FormType.DOUBLE);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid double 0 length1", d);
        assertAllInvalid(v2, sm, FormType.DOUBLE);

        // validate 5 max length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid double 5 length1", null);
        sm.putState("valid double 5 length2", d);
        assertAllValid(v3, sm, FormType.DOUBLE);


        // ilc_022502.13_end

        // ilc_022502.14_start
        // use StateMap to test
        /*
        //v1
        assertValid("Error validating null.length()<=-1", v1, el, null);
        assertInvalid("Error invalidating d.length()<=-1", v1, el, d);
        
        //v2
        assertValid("Error validating null.length()<=0", v2, el, null);
        assertInvalid("Error invalidating d.length()<=0", v2, el, d);
        
        //v3
        assertValid("Error validating null.length()<=5", v3, el, null);
        assertValid("Error validating d.length()<=5", v3, el, d);
        */
        // ilc_022502.14_end
    }

    /**
     * Test Float types
     */
    public void testFloat() {
        //create a form validator and excercise it
        MaxLengthValidator v1 = new MaxLengthValidator(-1);
        MaxLengthValidator v2 = new MaxLengthValidator(0);
        MaxLengthValidator v3 = new MaxLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.FLOAT);
        Float f = new Float(1.23);

        // ilc_022502.15_start
        // use a StateMap for testing to make sure elements go through mapping

        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 max length
        sm.putState("valid float -1 length", null);
        assertAllValid(v1, sm, FormType.FLOAT);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid float -1 length1", f);
        assertAllInvalid(v1, sm, FormType.FLOAT);

        // validate 0 max length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid float 0 length1", null);
        assertAllValid(v2, sm, FormType.FLOAT);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid float 0 length1", f);
        assertAllInvalid(v2, sm, FormType.FLOAT);

        // validate 5 max length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid float 5 length1", null);
        sm.putState("valid float 5 length2", f);
        assertAllValid(v3, sm, FormType.FLOAT);


        // ilc_022502.15_end

        // ilc_022502.16_start
        // use StateMap to test
        /*
        //v1
        assertValid("Error validating null.length()<=-1", v1, el, null);
        assertInvalid("Error invalidating f.length()<=-1", v1, el, f);
        
        //v2
        assertValid("Error validating null.length()<=0", v2, el, null);
        assertInvalid("Error invalidating f.length()<=0", v2, el, f);
        
        //v3
        assertValid("Error validating null.length()<=5", v3, el, null);
        assertValid("Error validating f.length()<=5", v3, el, f);
        */
        // ilc_022502.16_end
    }
    
}
