/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestViewUtil.java,v 1.8 2002/11/22 23:10:23 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.view;

import java.io.*;
import java.util.*;
import java.security.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.plankton.*;
import org.enhydra.barracuda.plankton.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.*;
import org.enhydra.barracuda.testbed.*;
import org.enhydra.barracuda.testbed.servlet.*;


/**
 * This test verifies that the ViewUtil class can parse
 * request headers to validate client capabilities
 */
public class TestViewUtil extends DefaultTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestViewUtil.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);
    public static final String UAS_PROPS = "user-agents.properties";

    //variables
    String uas = null;
    ClientType ctTarget = null;

    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestViewUtil(String name) {
        super(name);
    }

    /**
     * Public Constructor
     */
    public TestViewUtil(String name, String iuas, ClientType ictTarget) {
        super(name);
        uas = iuas;
        ctTarget = ictTarget;
    }

    /**
     * Every test class should have a main method so it can be run
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    /**
     * Because we need to test so many different user-agent strings,
     * we will return a suite of tests...
     */
	public static Test suite() {
        //create the test suite
		TestSuite suite = new TestSuite();
        
        //this test fundamentally depends on the servlet wrapper object,
        //so we need to test that as well
        suite.addTest(new TestViewUtil("testMockup"));

        //start by getting a reference to the properties file
        InputStream is = TestViewUtil.class.getResourceAsStream(UAS_PROPS);
        BufferedReader in = new BufferedReader(new InputStreamReader(is));
        boolean inComments = false;
        try {
            while (true) {
                //read a line & make sure its valid
                String s = in.readLine();
                if (s==null) break;
                s = s.trim();
                if (s.startsWith("//")) continue;
                if (!inComments) {
                    int cpos = s.indexOf("/*");
                    if (cpos>-1) {
                        s = s.substring(0,cpos);
                        inComments = true;
                    }
                } else {
                    int cpos = s.indexOf("*/");
                    if (cpos>-1) {
                        s = s.substring(cpos+2);
                        inComments = false;
                    } else {
                        continue;
                    }
                }
                if (s.equals("")) continue;
                
                //now figure out the uas and ctype settings
                String uas = null;
                String cts = null;
                int spos = s.indexOf("==");
                if (spos==-1) throw new RuntimeException("Fatal error parsing "+UAS_PROPS+" --> "+s);
                uas = s.substring(0,spos).trim();
                if (uas.equals("")) throw new RuntimeException("Invalid user-agent string in "+UAS_PROPS+" --> "+s);
                cts = s.substring(spos+2).trim();
                if (cts==null) throw new RuntimeException("Fatal error parsing client type "+UAS_PROPS+" --> "+s);
                ClientType ct = ClientType.getInstance(cts);
                if (ct==null) throw new RuntimeException("Invalid client type in "+UAS_PROPS+" --> "+cts);
     
                //add the test
                suite.addTest(new TestViewUtil("testClientType", uas, ct));
            }
        } catch (IOException e) {}
        
        //finally return the suite
        return suite;
    }

    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!


    /**
     * Test the client types to make sure we get what we expect for a given 
     * user agent string
     */
    public void testClientType() {
        if (logger.isInfoEnabled()) logger.info("testing ViewUtil.getClientType: uas="+uas);
        MockHttpServletRequest req = new MockHttpServletRequest(null);
        req.setHeader("User-Agent", uas);
        ClientType ct = ViewUtil.getClientType(req);
        assertTrue("null value returned for user-agent='"+uas+"'", ct!=null);
        assertTrue("returned '"+ct+"' instead of '"+ctTarget+"' for user-agent='"+uas, ct==ctTarget);
    }

    /**
     * Simple test to make sure the mockup servlet request to
     * make sure it functions as expected (ie. the necessary methods
     * do what the wrapper class will expect them to do)
     */
    public void testMockup() {
        if (logger.isInfoEnabled()) logger.info("testing servlet request mockup");

        //vars
        MockHttpServletRequest req = null;
        Enumeration enum = null;
        String k1 = "foo1";
        String k2 = "foo2";
        String s1 = "blah1";
        String s2a = "blah2.a";
        String s2b = "blah2.b";

        //try something simple first (to make sure our mockup is working)
        //...null param string
        req = new MockHttpServletRequest(null);
        assertTrue("mockup check 1a failed - hdr map not converted correctly", req.hdrMap.size()==0);
        assertTrue("mockup check 1b.1 failed - returned bad value", req.getHeader(k1)==null);
        enum = req.getHeaderNames();
        assertTrue("mockup check 1c.1 failed - returned null or non-empty enum", enum!=null && !enum.hasMoreElements());
        enum = req.getHeaders("foo");
        assertTrue("mockup check 1d.1 failed - returned null or non-empty enum", enum!=null && !enum.hasMoreElements());
        //...now try with some data
        req = new MockHttpServletRequest(null);
        req.setHeader(k1,s1);
        req.addHeader(k2,s2a);
        req.addHeader(k2,s2b);
        assertTrue("mockup check 2a failed - hdr map not converted correctly", req.hdrMap.size()==2);
        assertTrue("mockup check 2b.1 failed - returned bad value", "blah1".equals(req.getHeader(k1)));
        enum = req.getHeaderNames();
        for (int i=0; i<2; i++) {
            Object o = enum.nextElement();
            assertTrue("mockup check 2c.1 failed - wrong element, cntr="+i, o==k1 || o==k2);
        }
        assertTrue("mockup check 2c.2 failed - enum claims more elements", !enum.hasMoreElements());
        assertTrue("mockup check 2d failed - wrong value", s1.equals(req.getHeader(k1)));
        enum = req.getHeaders(k2);
        for (int i=0; i<2; i++) {
            Object o = enum.nextElement();
            assertTrue("mockup check 2e.2 failed - wrong element, cntr="+i, o==s2a || o==s2b);
        }
        assertTrue("mockup check 2e.2 failed - enum claims more elements", !enum.hasMoreElements());
    }

}
