/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: TestStringUtil.java,v 1.3 2002/12/01 18:19:09 JacobK Exp $
 */
package org.enhydra.barracuda.plankton;

import java.util.*;
// jrk_2002-12-01.1 - the sql libraries are unused.  No sense in importing them.
//import java.sql.*;
//import javax.sql.*;

import junit.framework.*;
//import org.enhydra.barracuda.plankton.data.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.testbed.*;


/**
 * Test the StringUtil class
 */
public class TestStringUtil extends DefaultTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestStringUtil.class.getName();
    private static final Logger logger = Logger.getLogger("test."+testClass);

    //variables
    protected String sourceStr = null;
    protected String oldPattern = null;
    protected String newPattern = null;
    protected String expectedResult = null;
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestStringUtil(String name) {
        super(name);
//logger.setPriority(org.apache.log4j.Priority.DEBUG);
//com.atmr.kilimanjaro.nxgenw.sql.RepSQL.logger.setPriority(org.apache.log4j.Priority.DEBUG);
    }

    /**
     * Public Constructor
     */
    public TestStringUtil(String name, String isourceStr, String ioldPattern, String inewPattern, String iexpectedResult) {
        super(name);
        sourceStr = isourceStr;
        oldPattern = ioldPattern;
        newPattern = inewPattern;
        expectedResult = iexpectedResult;
    }

    /**
     * Every test class should have a main method so it can be run
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    /**
     * Return a suite of tests...
     */
    public static Test suite() {
        //create the test suite
        TestSuite suite = new TestSuite();
        
        //add the tests
        suite.addTest(new TestStringUtil("testReplace", null, null, "Blah", "Blah"));
        suite.addTest(new TestStringUtil("testReplace", null, "a", "Blah", null));
        suite.addTest(new TestStringUtil("testReplace", "Foo Blah", null, "_", "Foo Blah"));
        suite.addTest(new TestStringUtil("testReplace", "Foo Blah", " ", "_", "Foo_Blah"));
        suite.addTest(new TestStringUtil("testReplace", "Foo Blah Blah", " ", "_", "Foo_Blah_Blah"));
        suite.addTest(new TestStringUtil("testReplace", "Foo  Blah", " ", "_", "Foo__Blah"));
        suite.addTest(new TestStringUtil("testReplace", "~Some Text", "~", null, "Some Text"));
        suite.addTest(new TestStringUtil("testReplace", "Some Text~", "~", null, "Some Text"));
        suite.addTest(new TestStringUtil("testReplace", "~Some~Text~", "~", null, "SomeText"));
        suite.addTest(new TestStringUtil("testReplace", "~~~", "~", null, null));
        suite.addTest(new TestStringUtil("testReplace", "~~~", "~", "", ""));
        suite.addTest(new TestStringUtil("testReplace", "", " ", null, ""));

        //finally return the suite
        return suite;
    }

    //-------------------- Actual Tests --------------------------
    /**
     * Test a table to make sure it contains the expected columns
     */
    // jrk_2002-12-01.2 - the sql libraries are unused.  No sense in referencing them.
    //public void testReplace() throws SQLException {
    public void testReplace() {
        logger.info("running testReplace()");

        String actualResult = StringUtil.replace(sourceStr, oldPattern, newPattern);
        assertTrue("Replace err on source:'"+sourceStr+"' ('"+oldPattern+"'-->'"+newPattern+"') got:'"+actualResult+"' expected:'"+expectedResult+"'", actualResult==expectedResult || (actualResult!=null && actualResult.equals(expectedResult)));
    }
}
