/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestFileNames.java,v 1.7 2003/05/16 13:39:39 jacobk Exp $
 */
package org.enhydra.barracuda.taskdefs;

// import junit specifics
import junit.framework.TestCase;

// import logging specifics
import org.apache.log4j.Logger;

// import java specifics
import java.io.File;
import java.io.IOException;

import org.enhydra.xml.xmlc.taskdef.*;

import org.enhydra.barracuda.testbed.*;

/**
 * Unit tests for buildFullBaseFileName() and buildClassName() methods
 * of DefaultXmlcUtilsImpl
 */
public class TestFileNames extends DefaultTestCase {

    //===========================================================================
    // CONSTRUCTORS
    //===========================================================================

    /**
     * Default constructor
     *
     * @param theName   The name of the test
     */
    public TestFileNames( String theName) {
        super( theName );
    }

    /**
     * Sets up the fixture, for example, open a network connection.
     * This method is called before a test is executed.
     */
    protected void setUp() throws Exception {
        super.setUp();
    }

    /**
     * Tears down the fixture, for example, close a network connection.
     * This method is called after a test is executed.
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    //===========================================================================
    // TEST METHODS
    //===========================================================================
    public void testBasicBuildFullBaseFileName() throws IOException {
        XmlcUtils xmlcUtils = XmlcUtils.create();

        String baseFilename = "base";
        String newBaseFileName = xmlcUtils.buildFullBaseFileName( null, null, baseFilename );
        assertEquals("Name should be unchanged", baseFilename, newBaseFileName );

        baseFilename = "base.html";
        String expectedBaseFileName = "base";
        newBaseFileName = xmlcUtils.buildFullBaseFileName( null, null, baseFilename );
        assertEquals("Type should be stripped off", expectedBaseFileName, newBaseFileName );
    }

    public void testPackageNameBuildFullBaseFileName() throws IOException {
        XmlcUtils xmlcUtils = XmlcUtils.create();

        String baseFilename = "base";

        String packageName = "a";
        String expectedBaseFileName = "a/" + baseFilename;
        String newBaseFileName = xmlcUtils.buildFullBaseFileName( packageName, null, baseFilename );
        assertEquals("Name 0 should be as expected",
                     expectedBaseFileName,
                     newBaseFileName );

        packageName = "";
        expectedBaseFileName = baseFilename;
        newBaseFileName = xmlcUtils.buildFullBaseFileName( packageName, null, baseFilename );
        assertEquals("Name should be unchanged",
                     expectedBaseFileName,
                     newBaseFileName );

        packageName = "a.b.c.d";
        expectedBaseFileName = "a/b/c/d/" + baseFilename;
        newBaseFileName = xmlcUtils.buildFullBaseFileName( packageName, null, baseFilename );
        assertEquals("Name 1 should be as expected",
                     expectedBaseFileName,
                     newBaseFileName );

        packageName = "p1.p2.p3.p4.p5.";
        expectedBaseFileName = "p1/p2/p3/p4/p5/" + baseFilename;
        newBaseFileName = xmlcUtils.buildFullBaseFileName( packageName, null, baseFilename );
        assertEquals("Name 2 should be as expected",
                     expectedBaseFileName,
                     newBaseFileName );

        baseFilename = "/test1/test2/test3/base";
        packageName = "";
        expectedBaseFileName = baseFilename;
        newBaseFileName = xmlcUtils.buildFullBaseFileName( packageName, null, baseFilename );
        assertEquals("Name should be unchanged again",
                     expectedBaseFileName,
                     newBaseFileName );

        packageName = "a.b.c.d.";
        expectedBaseFileName = "a/b/c/d" + baseFilename;
        newBaseFileName = xmlcUtils.buildFullBaseFileName( packageName, null, baseFilename );
        assertEquals("Name 3 should be as expected",
                     expectedBaseFileName,
                     newBaseFileName );
    }

    public void testPackageDirBuildFullBaseFileName() throws IOException {
        XmlcUtils xmlcUtils = XmlcUtils.create();

        String baseFilename = "base";
        String packageDir = "a";
        String expectedBaseFileName = "a/" + baseFilename;
        String newBaseFileName = xmlcUtils.buildFullBaseFileName( null, packageDir, baseFilename );
        assertEquals("Name 0 should be as expected",
                     expectedBaseFileName,
                     newBaseFileName );

        baseFilename = "test.base";
        packageDir = "";
        expectedBaseFileName = "test";
        newBaseFileName = xmlcUtils.buildFullBaseFileName( null, packageDir, baseFilename );
        assertEquals("Name 1 should be as expected",
                     expectedBaseFileName,
                     newBaseFileName );

        baseFilename = "\\base";
        packageDir = "a/";
        expectedBaseFileName = "a/" + "base";
        newBaseFileName = xmlcUtils.buildFullBaseFileName( null, packageDir, baseFilename );
        assertEquals("Name 2 should be as expected",
                     expectedBaseFileName,
                     newBaseFileName );

        baseFilename = "/test1/test2/test3/base";
        packageDir = "a/b/c/d";
        expectedBaseFileName = "a/b/c/d/" + "base";
        newBaseFileName = xmlcUtils.buildFullBaseFileName( null, packageDir, baseFilename );
        assertEquals("Name 3 should be as expected",
                     expectedBaseFileName,
                     newBaseFileName );

        baseFilename = "test1\\test2/test3\\base/";
        packageDir = "\\a/b\\c/d\\";
        expectedBaseFileName = "/a/b/c/d/" + "base";
        newBaseFileName = xmlcUtils.buildFullBaseFileName( null, packageDir, baseFilename );
        assertEquals("Name 4 should be as expected",
                     expectedBaseFileName,
                     newBaseFileName );
    }

    public void testBuildFullBaseFileName() throws IOException {
        XmlcUtils xmlcUtils = XmlcUtils.create();

        String baseFilename = "base";
        String packageName = "packageName";
        String packageDir = "packageDir";
        String expectedBaseFileName = packageName + "/" + packageDir + "/" + baseFilename;
        String newBaseFileName = xmlcUtils.buildFullBaseFileName( packageName, packageDir, baseFilename );
        assertEquals("Name 0 should be as expected",
                     expectedBaseFileName,
                     newBaseFileName );

        baseFilename = "/dir1/dir2/dir3/base.test.test";
        packageName = "name1.name2.name3....";
        packageDir = "/dirA/dirB/dirc/";
        expectedBaseFileName = "name1/name2/name3" + packageDir + "base";
        newBaseFileName = xmlcUtils.buildFullBaseFileName( packageName, packageDir, baseFilename );
        assertEquals("Name 1 should be as expected",
                     expectedBaseFileName,
                     newBaseFileName );

        baseFilename = "/dir1/dir2/dir3/base.test.test";
        packageName = "name1.name2.name3....";
        packageDir = "";
        expectedBaseFileName = "name1/name2/name3/" + "dir1/dir2/dir3/base";
        newBaseFileName = xmlcUtils.buildFullBaseFileName( packageName, packageDir, baseFilename );
        assertEquals("Name 2 should be as expected",
                     expectedBaseFileName,
                     newBaseFileName );
    }

    public void testBuildClassName() throws IOException {
        XmlcUtils xmlcUtils = XmlcUtils.create();

        String filename = "p1/p2/p3/base";
        String typeModifier = "HTML";
        String expectedClassName = "p1.p2.p3.baseHTML";
        String newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 0 should be as expected",
                     expectedClassName,
                     newClassName );
    }


    //===========================================================================
    // DATA MEMBERS
    //===========================================================================

    //===========================================================================
    // STATIC DATA MEMBERS
    //===========================================================================

    // setup the loggger for this class
    private static final Logger theirLogger = Logger.getLogger( TestFileNames.class.getName() );

}
